@@ -48,6 +48,8 @@
 #include "common/version.h"
 #include "main.h"
 #include "plugin.h"
+#include "mainwindow.h"
+#include "summaryview.h"
 #include "mimeview.h"
 #include "messageview.h"
 #include "prefs_common.h"
@@ -240,6 +242,13 @@ static gboolean has_header(FILE *fp)
 	return FALSE;
 }
 
+static void gtkhtml2_restore_focus(void)
+{
+	MainWindow *mainwin = mainwindow_get_mainwindow();
+	if (mainwin)
+		summary_grab_focus(mainwin->summaryview);
+}
+
 static gint gtkhtml2_show_mimepart_real(MimeViewer *_viewer)
 {
 	GtkHtml2Viewer *viewer = (GtkHtml2Viewer *) _viewer;
@@ -255,7 +264,8 @@ static gint gtkhtml2_show_mimepart_real(MimeViewer *_viewer)
 	memset(buf, 0, sizeof(buf));
 
 	viewer->loading = 1;
-	messageview->updating = TRUE;
+	if (messageview)
+		messageview->updating = TRUE;
 	debug_print("gtkhtml2_show_mimepart\n");
 
 	if (viewer->filename != NULL) {
@@ -369,7 +379,8 @@ out:
 	viewer->stop_previous = FALSE;
 	viewer->force_image_loading = FALSE;	
 	g_mutex_unlock(viewer->mutex);
-	messageview->updating = FALSE;
+	if (messageview)
+		messageview->updating = FALSE;
 	viewer->preparing = FALSE;
 	return FALSE;
 }
@@ -419,6 +430,7 @@ static void reload_with_img(NoticeView *noticeview, GtkHtml2Viewer *viewer)
 {
 	viewer->force_image_loading = TRUE;
 	gtkhtml2_show_mimepart((MimeViewer *)viewer, NULL, viewer->mimeinfo);
+	gtkhtml2_restore_focus();
 }
 #endif
 
@@ -550,6 +562,7 @@ void link_clicked(HtmlDocument *doc, const gchar *url, GtkHtml2Viewer *viewer) {
 			       bevent->button, bevent->time);
 	}
 	g_free(real_url);
+	gtkhtml2_restore_focus();
 }
 
 
@@ -559,7 +572,6 @@ void link_clicked(HtmlDocument *doc, const gchar *url, GtkHtml2Viewer *viewer) {
  */
 static void on_url(GtkWidget *widget, const gchar *url, gpointer data)
 {
-	gchar *trimmed_uri = NULL;
 	MimeViewer *viewer = (MimeViewer *)data;
 	MessageView *messageview = viewer->mimeview ? viewer->mimeview->messageview : NULL;
 
@@ -567,12 +579,10 @@ static void on_url(GtkWidget *widget, const gchar *url, gpointer data)
 
 	if (url != NULL) {
 		gchar *real_url = make_url(url, ((GtkHtml2Viewer *)data)->base);
-		trimmed_uri = trim_string(real_url?real_url:url, 60);
 		if (messageview->statusbar)
 			gtk_statusbar_push(GTK_STATUSBAR(messageview->statusbar),
-				   messageview->statusbar_cid, trimmed_uri);
+				   messageview->statusbar_cid, real_url);
 		g_free(real_url);
-		g_free(trimmed_uri);
 		((GtkHtml2Viewer *)viewer)->is_on_url = TRUE;
 	} else {
 		if (messageview->statusbar)
@@ -620,6 +630,7 @@ static void *gtkhtml_fetch_feed_threaded(void *arg)
 		ctx->ready = TRUE;
 		claws_unlink(template);
 		g_free(template);
+		fclose(f);
 		return NULL;
 	}
 
@@ -721,8 +732,6 @@ static void requested_url(HtmlDocument *doc, const gchar *url, HtmlStream *strea
 		gchar *cache_file = NULL;
 #endif
 		debug_print("looking for %s in Content-Location\n", url);
-                if (url == '\0')
-                        goto fail;
 
                 while ((mimeinfo = procmime_mimeinfo_next(mimeinfo)) != NULL) {
                         if (mimeinfo->location != NULL && strcmp(mimeinfo->location, url) == 0)
